/* BFEncodingUtilities.h created by lindberg on Fri 28-Jan-2000 */

#import <Foundation/NSObjCRuntime.h>
//#import "FrameworkDefines.h"

@class NSString, NSData;

/*"
 * Functions to do Base-64 (RFC 1521) encoding and decoding of data.
 * 
 * BFBase64EncodeBytes() encodes the given bytes, returning the resulting
 * Base-64 buffer. The caller is responsible for NSZoneFree()ing the return
 * value. The size of the returned buffer is returned by reference in outLength.
 * If newlineString is non-NULL, then that string will be inserted for every 72
 * characters of Base-64 data generated. BFBase64EncodeData() and
 * BFBase64EncodeString() are variations to encode NSData or NSString values,
 * returning an autoreleased NSString object.
 * 
 * BFBase64DecodeBytes() decodes the given bytes, return the resulting data
 * buffer. The caller is responsible for NSZoneFree()ing the return value. The
 * size of the decoded data is returned by reference in outLength (though the
 * allocated size of the buffer may be larger). Whitespace and invalid base64
 * characters are ignored, but if the data ends prematurely an exception is
 * raised. BFBase64DecodeData() and BFBase64DecodeString() are variations to
 * return autoreleased NSData and NSString objects.
"*/
char     *BFBase64DecodeBytes(const char *bytes, int length, int *outLength);
char     *BFBase64EncodeBytes(const char *bytes, int length, const char *newlineString, int *outLength);

NSString *BFBase64EncodeData(NSData *data, NSString *newlineString);
NSString *BFBase64EncodeString(NSString *string, NSString *newlineString);
NSData   *BFBase64DecodeData(NSString *string);
NSString *BFBase64DecodeString(NSString *string);

/*"
 * ASCII85 encoding/decoding. ASCII85 is similar in intent to
 * Base64/Uuencoding/Xxencoding, in that it converts binary data to ASCII and
 * back. However, it converts four bytes of binary data to five bytes of ASCII
 * data, meaning it results in smaller output than the other methods (which
 * encode three bytes of binary into four bytes of ASCII). It was developed by
 * Adobe for use in PostScript and PDF for situations where normally binary data
 * needed to be converted to ASCII (for sending through email, etc.).
 * 
 * BFASCII85EncodeBytes() encodes the given bytes, returning the resulting
 * ASCII85 buffer. The caller is responsible for NSZoneFree()ing the return
 * value. The size of the returned buffer is returned by reference in outLength.
 * If newlineString is non-NULL, then that string will be inserted for (roughly)
 * every 72 characters of ASCII85 data generated. If suffix is YES, then the
 * end-of-data marker "~>" will be appended. BFASCII85EncodeData() and
 * BFASCII85EncodeString() are variations to encode NSData or NSString values,
 * returning an autoreleased NSString object.
 * 
 * BFASCII85DecodeBytes() decodes the given bytes, returning a memory buffer
 * that the caller is responsible for freeing. It will ignore any whitespace
 * characters and handle the end-of-data marker "~>" if present, but any other
 * illegal characters will cause exceptions to be raised. BFASCII85DecodeData()
 * and BFASCII85DecodeString() are variations to return autoreleased NSData or
 * NSString objects.
"*/
char *BFASCII85EncodeBytes(const char *bytes, int length, BOOL suffix, const char *newlineString, int *outLength);
char *BFASCII85DecodeBytes(const char *bytes, int length, int *outLength);

NSData   *BFASCII85DecodeData(NSString *string);
NSString *BFASCII85DecodeString(NSString *string);
NSString *BFASCII85EncodeData(NSData *data, BOOL suffix, NSString *newline);
NSString *BFASCII85EncodeString(NSString *data, BOOL suffix, NSString *newline);

/*"
 * Hexadecimal encoding/decoding.
 * 
 * BFDataFromHexString() decodes the hexidecmial data in hexString, and returns
 * the corresponding raw binary data. Prefixes of "0x" and "0X" will be properly
 * ignored, but there can be no spaces or other illegal characters in hexString.
 * 
 * BFHexStringFromData() generates a hex string from the given data. If
 * %uppercase is YES, then the hex data will use uppercase letters for the 10-15
 * values, lowercase otherwise. If %usePrefix is YES, then the returned string
 * will have a "0x" prefix. If %spacing is non-zero, then a space character will
 * be inserted in the output after every %spacing input bytes. For example, if
 * %spacing is 4, then a space will be inserted after every 4 input bytes, i.e.
 * every 8 hex digits. BFHexStringFromBytes() is a version that takes raw bytes
 * instead of an NSData object.
"*/
NSData   *BFDataFromHexString(NSString *hexString);
NSString *BFHexStringFromData(NSData *data, BOOL usePrefix, BOOL uppercase, int spacing);
NSString *BFHexStringFromBytes(const void *bytes, int length, BOOL usePrefix, BOOL uppercase, int spacing);

/*"
 * Message digest functions to get a signature for an NSData object or raw
 * bytes. The signature is essentially unique for any given block of data. These
 * hash functions are often used for digital signatures. Many other hash
 * functions exist; these are just the two most common.
 * 
 * MD5 (RFC 1321) was developed by RSA Data Security, Inc. Recent literature
 * suggests that MD5 may be weaker than initially thought; successful attacks
 * have been found on its predecessor (MD4) and on portions of MD5 as well.
 * While no successful attacks have been found to date against the full MD5, the
 * existing attacks are troublesome, and therefore it is not recommended to use
 * MD5 for secure applications. The length of the signature returned is 128
 * bits, or 16 bytes.
 * 
 * SHA-1, or Secure Hash Algorithm (NIST FIPS 180-1), was developed by the NIST
 * and the NSA. It fixes an unpublished flaw in the earlier SHA algorithm, and
 * is the NIST standard hash function for use in secure applications. The length
 * of the signature returned is 160 bits, or 20 bytes.
"*/
NSData *BFMD5SignatureForData(NSData *data);
NSData *BFMD5SignatureForBytes(const void *bytes, int length);
NSData *BFSHA1SignatureForData(NSData *data);
NSData *BFSHA1SignatureForBytes(const void *bytes, int length);

/*"
 * These functions implement a simple password system. BFCryptPasswordWithSalt()
 * takes a password of any length and a two-byte salt, prepends the salt to the
 * password, and gets the SHA-1 signature of the result. SHA-1 is a one-way hash
 * function, so there is no way to guess the original password from the hashed
 * data. The first 10 bytes of the hash data is appended to the salt, creating a
 * 12-byte piece of data. This is then Base64-encoded to return a 16-byte ASCII
 * string. BFCryptPassword() just calls BFCryptPasswordWithSalt() with a random
 * two-byte salt (using BFRandom). The purpose of the salt is simply so that
 * identical passwords with different salts end up as completely different data,
 * so that someone looking at crypted data couldn't tell if two people had the
 * same passwords.
 * 
 * BFPasswordIsEqual() sees if a potential password is equal to a
 * previously-crypted password. It takes %cryptPassword (which should have
 * originally been returned from BFCryptPasswordWithSalt()), Base64-decodes it,
 * and uses the salt from the original to crypt the %tryPassword. If the
 * resulting data is the same as the original, then the passwords are a match.
"*/
NSString *BFCryptPasswordWithSalt(NSString *password, unsigned char salt[2]);
NSString *BFCryptPassword(NSString *password);
BOOL BFPasswordIsEqual(NSString *tryPassword, NSString *cryptPassword);
